"use strict";

const {
  URL
} = require(`url`);

const {
  createRemoteFileNode
} = require(`gatsby-source-filesystem`);

const path = require(`path`);

const probeImageSize = require(`probe-image-size`);

const {
  getOptions
} = require(`./plugin-options`);

const getHref = link => {
  if (typeof link === `object`) {
    return link.href;
  }

  return link;
};

exports.getHref = getHref;

const nodeFromData = async (datum, createNodeId, entityReferenceRevisions = [], pluginOptions) => {
  const {
    attributes: {
      id: attributeId,
      ...attributes
    } = {}
  } = datum;
  const preservedId = typeof attributeId !== `undefined` ? {
    _attributes_id: attributeId
  } : {};
  const langcode = attributes.langcode || `und`;
  const type = datum.type.replace(/-|__|:|\.|\s/g, `_`);
  const isFile = isFileNode(datum, type);
  const url = isFile ? pluginOptions.baseUrl + getFileUrl(datum.attributes) : null;
  const imageSize = isFile ? await probeImageSize(url) : {};
  const gatsbyImageCdnFields = isFile ? {
    filename: attributes === null || attributes === void 0 ? void 0 : attributes.filename,
    url,
    placeholderUrl: url,
    width: imageSize.width,
    height: imageSize.height,
    mimeType: datum.attributes.filemime
  } : {};
  return {
    id: createNodeId(createNodeIdWithVersion(datum.id, datum.type, langcode, attributes.drupal_internal__revision_id, entityReferenceRevisions)),
    drupal_id: datum.id,
    parent: null,
    drupal_parent_menu_item: attributes.parent,
    children: [],
    ...attributes,
    ...preservedId,
    ...gatsbyImageCdnFields,
    drupal_relationships: datum.relationships,
    relationships: {},
    internal: {
      type
    }
  };
};

exports.nodeFromData = nodeFromData;

const isEntityReferenceRevision = (type, entityReferenceRevisions = []) => entityReferenceRevisions.findIndex(revisionType => type.indexOf(revisionType) === 0) !== -1;

const createNodeIdWithVersion = (id, type, langcode, revisionId, entityReferenceRevisions = []) => {
  var _getOptions, _getOptions$languageC, _getOptions$languageC2;

  // Fallback to default language for entities that don't translate.
  if ((_getOptions = getOptions()) !== null && _getOptions !== void 0 && (_getOptions$languageC = _getOptions.languageConfig) !== null && _getOptions$languageC !== void 0 && _getOptions$languageC.nonTranslatableEntities.includes(type)) {
    langcode = getOptions().languageConfig.defaultLanguage;
  } // If the source plugin hasn't enabled `translation` then always just set langcode
  // to "undefined".


  let langcodeNormalized = getOptions().languageConfig ? langcode : `und`;

  if (getOptions().languageConfig && !((_getOptions$languageC2 = getOptions().languageConfig) !== null && _getOptions$languageC2 !== void 0 && _getOptions$languageC2.enabledLanguages.includes(langcodeNormalized))) {
    langcodeNormalized = getOptions().languageConfig.defaultLanguage;
  } // The relationship between an entity and another entity also depends on the revision ID if the field is of type
  // entity reference revision such as for paragraphs.


  return isEntityReferenceRevision(type, entityReferenceRevisions) ? `${langcodeNormalized}.${id}.${revisionId || 0}` : `${langcodeNormalized}.${id}`;
};

exports.createNodeIdWithVersion = createNodeIdWithVersion;

const isFileNode = (node, inputType) => {
  var _node$internal;

  const type = inputType || (node === null || node === void 0 ? void 0 : (_node$internal = node.internal) === null || _node$internal === void 0 ? void 0 : _node$internal.type);
  return type === `files` || type === `file__file`;
};

exports.isFileNode = isFileNode;

const getFileUrl = node => {
  let fileUrl = node.url;

  if (typeof node.uri === `object`) {
    // Support JSON API 2.x file URI format https://www.drupal.org/node/2982209
    fileUrl = node.uri.url;
  }

  return fileUrl;
};

exports.downloadFile = async ({
  node,
  store,
  cache,
  createNode,
  createNodeId,
  getCache,
  reporter
}, {
  basicAuth,
  baseUrl
}) => {
  // handle file downloads
  if (isFileNode(node)) {
    let fileType;

    if (typeof node.uri === `object`) {
      // get file type from uri prefix ("S3:", "public:", etc.)
      const uriPrefix = node.uri.value.match(/^\w*:/);
      fileType = uriPrefix ? uriPrefix[0] : null;
    }

    const fileUrl = getFileUrl(node, baseUrl); // Resolve w/ baseUrl if node.uri isn't absolute.

    const url = new URL(fileUrl, baseUrl); // If we have basicAuth credentials, add them to the request.

    const basicAuthFileSystems = [`public:`, `private:`, `temporary:`];
    const auth = typeof basicAuth === `object` && basicAuthFileSystems.includes(fileType) ? {
      htaccess_user: basicAuth.username,
      htaccess_pass: basicAuth.password
    } : {};
    const fileNode = await createRemoteFileNode({
      url: url.href,
      name: path.parse(decodeURIComponent(url.pathname)).name,
      store,
      cache,
      createNode,
      createNodeId,
      getCache,
      parentNodeId: node.id,
      auth,
      reporter
    });

    if (fileNode) {
      node.localFile___NODE = fileNode.id;
    }
  }
};